//Copyright (c) 1999, Art Gittleman
//This example is provided WITHOUT ANY WARRANTY either expressed or implied.

/* Uses wait and notify to enable
 * producer and consumer threads to
 * cooperate in using a buffer.
 */

// Updated to concurrent by Dr. Spiegel
// Updated to use version 5 Synchronous queue to do rendezvous
//   Buffer gone; fair schedule...

import java.util.concurrent.*;
  
public class FairCPutGet {

  // Zero Buffer; supports rendezvous...
  static SynchronousQueue<Integer> Buffer;

  class Producer extends Thread {

     public void run() {
       for(int i=1; ; i++)
         try {
             Buffer.put(i);
	     System.out.println("Put "+i);
	     sleep(100);
           }catch (InterruptedException e){
              e.printStackTrace();
           }
     }
  }

  class Consumer extends Thread {

     public void run() {
      while (true)
         try {
           int i=Buffer.take();
	   System.out.println("Took "+i);
	   sleep(100);
         }catch (InterruptedException e) {
               e.printStackTrace();
         }
     }
  }

  public static void main(String[] args) {
    if (args.length!=2) {
	System.out.println("Usage: PutGet  <#Producers>  <#Consumers>");
        System.exit(0);
    }
    FairCPutGet pg = new FairCPutGet();
    Buffer=new SynchronousQueue(true);
    Producer p[] = new Producer[Integer.parseInt(args[0])];
    for (int pIdx=0;pIdx<Integer.parseInt(args[0]);pIdx++) {
      p[pIdx]=pg.new Producer();
      p[pIdx].start();
    }
    Consumer c[] = new Consumer[Integer.parseInt(args[1])];
    for (int cIdx=0;cIdx<Integer.parseInt(args[1]);cIdx++) {
      c[cIdx]=pg.new Consumer();
      c[cIdx].start();
    }
  }
}

